@@ -1,15 +1,25 @@
+import http
 import re
 
+from http import HTTPStatus
+
 import aiohttp
 import pytest
 import responses
 
-from quickbuild import AsyncQBClient, ContentType, QBClient, QBError
+from quickbuild import (
+    AsyncQBClient,
+    ContentType,
+    QBClient,
+    QBError,
+    QBServerError,
+)
+from quickbuild.exceptions import QBForbiddenError
 
 GET_VERSION_DATA = '6.0.9'
 
 
-USERS_XML = """<?xml version="1.0" encoding="UTF-8"?>
+USERS_XML = r"""<?xml version="1.0" encoding="UTF-8"?>
 
 <list>
   <com.pmease.quickbuild.model.User>
@@ -65,7 +75,6 @@ def test_sync_client():
         responses.GET,
         re.compile(r'.*/rest/version'),
         body=GET_VERSION_DATA,
-        status=200
     )
 
     try:
@@ -89,6 +98,8 @@ def test_sync_client_retry():
     # responses library does`t support Retry mock
     # https://github.com/getsentry/responses/issues/135
     # so, just cover code of retry constructor
+    statuses = [HTTPStatus.BAD_REQUEST, HTTPStatus.INTERNAL_SERVER_ERROR]
+
     client = QBClient(
         'http://server',
         'login',
@@ -96,11 +107,11 @@ def test_sync_client_retry():
         retry=dict(
             total=10,
             factor=1,
-            statuses=[400, 500],
+            statuses=statuses,
         )
     )
 
-    assert client.session.adapters['http://'].max_retries.status_forcelist == [400, 500]
+    assert client.session.adapters['http://'].max_retries.status_forcelist == statuses
 
 
 @pytest.mark.asyncio
@@ -117,7 +128,7 @@ async def test_async_client(aiohttp_mock):
             'http://server/rest/version',
             content_type='text/plain',
             body=GET_VERSION_DATA,
-            status=200,
+            status=HTTPStatus.OK,
         )
 
         version = await client.system.get_version()
@@ -136,7 +147,7 @@ async def test_async_client_retry(aiohttp_mock):
         'password',
         retry=dict(
             total=10,
-            statuses=[500],
+            statuses=[HTTPStatus.INTERNAL_SERVER_ERROR],
         )
     )
 
@@ -144,14 +155,14 @@ async def test_async_client_retry(aiohttp_mock):
         'http://server/rest/version',
         content_type='text/plain',
         body='Server error',
-        status=500,
+        status=HTTPStatus.INTERNAL_SERVER_ERROR,
     )
 
     aiohttp_mock.get(
         'http://server/rest/version',
         content_type='text/plain',
         body=GET_VERSION_DATA,
-        status=200,
+        status=HTTPStatus.OK,
     )
 
     version = await client.system.get_version()
@@ -169,7 +180,7 @@ async def test_async_client_retry_exception(aiohttp_mock):
         'password',
         retry=dict(
             total=2,
-            statuses=[500],
+            statuses=[HTTPStatus.INTERNAL_SERVER_ERROR],
         )
     )
 
@@ -189,6 +200,14 @@ async def test_async_client_retry_exception(aiohttp_mock):
     await client.close()
 
 
+def test_retry_argument_validation():
+    with pytest.raises(QBError):
+        QBClient('http://server', retry=dict(total=1, strange_argument=1))
+
+    with pytest.raises(QBError):
+        AsyncQBClient('http://server', 'login', 'password', retry=dict(total=0))
+
+
 @responses.activate
 def test_content_type_parse_get():
     responses.add(
@@ -196,7 +215,6 @@ def test_content_type_parse_get():
         re.compile(r'.*/rest/users'),
         content_type='application/xml',
         body=USERS_XML,
-        status=200,
     )
 
     client = QBClient('http://server', content_type=ContentType.PARSE)
@@ -212,7 +230,6 @@ def test_content_type_xml_get():
         re.compile(r'.*/rest/users'),
         content_type='application/xml',
         body=USERS_XML,
-        status=200,
     )
 
     client = QBClient('http://server', content_type=ContentType.XML)
@@ -228,7 +245,6 @@ def test_content_type_json_get():
         re.compile(r'.*/rest/users'),
         content_type='application/json',
         body=USERS_JSON,
-        status=200,
     )
 
     client = QBClient('http://server', content_type=ContentType.JSON)
@@ -242,7 +258,7 @@ def test_content_type_headers_sync_json_json():
 
     def callback(request):
         assert request.headers['Accept'] == 'application/json'
-        return (200, request.headers, request.body)
+        return (HTTPStatus.OK, request.headers, request.body)
 
     responses.add_callback(
         responses.GET,
@@ -259,7 +275,7 @@ def test_content_type_headers_sync_xml_json():
 
     def callback(request):
         assert request.headers['Accept'] == 'application/json'
-        return (200, request.headers, request.body)
+        return (HTTPStatus.OK, request.headers, request.body)
 
     responses.add_callback(
         responses.GET,
@@ -276,7 +292,7 @@ def test_content_type_headers_sync_json_xml():
 
     def callback(request):
         assert request.headers['Accept'] == '*/*'
-        return (200, request.headers, request.body)
+        return (HTTPStatus.OK, request.headers, request.body)
 
     responses.add_callback(
         responses.GET,
@@ -286,3 +302,41 @@ def test_content_type_headers_sync_json_xml():
 
     client = QBClient('http://server', content_type=ContentType.JSON)
     client.memberships.get_info(1, content_type=ContentType.XML)
+
+
+@responses.activate
+def test_exception_server_error(client):
+    responses.add(
+        responses.GET,
+        re.compile(r'.*/rest/version'),
+        body=GET_VERSION_DATA,
+        status=HTTPStatus.INTERNAL_SERVER_ERROR
+    )
+
+    with pytest.raises(QBServerError):
+        client.system.get_version()
+
+
+@responses.activate
+def test_exception_forbidden_error(client):
+    responses.add(
+        responses.GET,
+        re.compile(r'.*/rest/version'),
+        body=GET_VERSION_DATA,
+        status=HTTPStatus.FORBIDDEN
+    )
+
+    with pytest.raises(QBForbiddenError):
+        client.system.get_version()
+
+
+@responses.activate
+def test_exception_common_error(client):
+    responses.add(
+        responses.GET,
+        re.compile(r'.*/rest/version'),
+        body=GET_VERSION_DATA,
+        status=HTTPStatus.METHOD_NOT_ALLOWED
+    )
+    with pytest.raises(QBError):
+        client.system.get_version()
